function simulate_and_save(savename,simtime)
%% simulate_and_save.m
% UAV Software-in-the-Loop simulate_and_save function
%
% This function runs the SIL sim and saves the results to a file with the
% given name. If no file name is given, "simData" is used. A simulation
% time can also be input; the default is 45 seconds.
%
% University of Minnesota 
% Aerospace Engineering and Mechanics 
% Copyright 2011 Regents of the University of Minnesota. 
% All rights reserved.
%
% SVN Info: $Id: simulate_and_save.m 337 2011-04-15 15:20:27Z murch $

if nargin < 1
    savename = 'simData'; % file name for sim data
end

if nargin < 2
    simtime = 45; % sec, simulation time
end

% Run simulation
sim('UAV_SIL', [0 simtime])

% Save the simulation data in the simData data structure.
simData.time= tout;
simData.elevator= elevator;
simData.aileron= aileron;
simData.rudder=rudder;
simData.flap= flap;
simData.V_s= V_s; 
simData.alt = alt; 
simData.phi=phi;
simData.theta = theta;
simData.psi = psi(:); % semicolon necessary to force variable usage instead of function call
simData.theta_cmd=theta_cmd;
simData.phi_cmd= phi_cmd;
simData.p=p;
simData.q=q; % Pitch rate
simData.r=r ;
simData.alpha = alpha(:); % semicolon necessary to force variable usage instead of function call
simData.beta = beta(:); % semicolon necessary to force variable usage instead of function call

% save data
save(savename,'simData');
