/******************************************************************************
 * FILE: rtalarms.c
 * DESCRIPTION:
 *
 *
 *
 *
 * AUTHOR: Rohit Pandita
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 * Copyright 2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: rtalarms.c 332 2011-04-14 18:43:14Z murch $
 ******************************************************************************/

#include "rtalarms.h"

void rt_alarms_init(void)
{
	int rtc_ticks_per_sec;

	/* get handle to the system realtime clock */
	rtclock = cyg_real_time_clock ();
	rtc_resolution = cyg_clock_get_resolution ( rtclock );

#ifdef verbose
	fprintf ( stderr, "cyg_realtime_clock() resolution: dividend=%d, \t divisor=%d", \
			rtc_resolution.dividend, rtc_resolution.divisor );
#endif
	rtc_ticks_per_sec = rtc_resolution.divisor;


	cyg_clock_to_counter ( rtclock, &rtc_counter );

	cyg_alarm_create ( rtc_counter,
			alarm_handlerfn_sensordaq,
			0,
			&alarm_hdl[0],
			&alarm_obj[0] );

	cyg_alarm_create ( rtc_counter,
			alarm_handlerfn_claw,
			0,
			&alarm_hdl[1],
			&alarm_obj[1] );

	cyg_alarm_create ( rtc_counter,
			alarm_handlerfn_nav,
			0,
			&alarm_hdl[2],
			&alarm_obj[2] );

	cyg_alarm_create ( rtc_counter,
			alarm_handlerfn_dnlink,
			0,
			&alarm_hdl[3],
			&alarm_obj[3] );

	cyg_alarm_create ( rtc_counter,
			alarm_handlerfn_datalogger,
			0,
			&alarm_hdl[4],
			&alarm_obj[4] );

	pthread_cond_init (&trigger_sensordaq, NULL);
	pthread_cond_init (&trigger_ahrs, NULL);
	pthread_cond_init (&trigger_claw, NULL);
	pthread_cond_init (&trigger_navigation, NULL);
	pthread_cond_init (&trigger_downlink, NULL);
	pthread_cond_init (&trigger_datalogger, NULL);

	// DAQ (50hz)
	threadstat[0].frequency = rtc_ticks_per_sec/DAQ_Hz;
	threadstat[1].frequency = rtc_ticks_per_sec/DAQ_Hz;		// AHRS freq
	cyg_alarm_initialize ( alarm_hdl[0], cyg_current_time(), threadstat[0].frequency );

	// CLAW (25hz)
	threadstat[3].frequency = rtc_ticks_per_sec/CLAW_Hz;
	cyg_alarm_initialize ( alarm_hdl[1], cyg_current_time(), threadstat[3].frequency );

	// NAV (10hz)
	threadstat[2].frequency = rtc_ticks_per_sec/NAV_Hz;
	cyg_alarm_initialize ( alarm_hdl[2], cyg_current_time(), threadstat[2].frequency );

	// DOWNLINK (10hz)
	threadstat[4].frequency = rtc_ticks_per_sec/DNLINK_Hz;
	cyg_alarm_initialize ( alarm_hdl[3], cyg_current_time(), threadstat[4].frequency );

	// DATALOGGER (20hz)
	threadstat[5].frequency = rtc_ticks_per_sec/DATALOGGER_Hz;
	cyg_alarm_initialize ( alarm_hdl[4], cyg_current_time(), threadstat[5].frequency );

}

/* alarm handler functions */
void alarm_handlerfn_sensordaq ( cyg_handle_t alarm, cyg_addrword_t data )
{
	pthread_cond_signal (&trigger_sensordaq);
}

void alarm_handlerfn_nav ( cyg_handle_t alarm, cyg_addrword_t data )
{
	pthread_cond_signal (&trigger_navigation);
}

void alarm_handlerfn_claw ( cyg_handle_t alarm, cyg_addrword_t data )
{
	pthread_cond_signal (&trigger_claw);
}

void alarm_handlerfn_dnlink ( cyg_handle_t alarm, cyg_addrword_t data )
{
	pthread_cond_signal (&trigger_downlink);
}

void alarm_handlerfn_datalogger ( cyg_handle_t alarm, cyg_addrword_t data )
{
	pthread_cond_signal (&trigger_datalogger);
}

/* destroy alarm objects and pthread_cond objects */
void rt_alarms_destroy (void)
{
	int i;

	for (i=0; i<NUM_ALARMS; i++)
		cyg_alarm_delete (alarm_hdl[i]);

	pthread_cond_destroy (&trigger_sensordaq);
	pthread_cond_destroy (&trigger_ahrs);
	pthread_cond_destroy (&trigger_navigation);
	pthread_cond_destroy (&trigger_claw);
	pthread_cond_destroy (&trigger_downlink);
	pthread_cond_destroy (&trigger_datalogger);
}
